#! /perl/bin/perl.exe

#*************************************************************************
# Author: Anthony A. Auerbach <tony_auerbach@hotmail.com>
# Date: 1/10/01
# Copyright: Anthony A. Auerbach 2001
#
# Descr: Iris Request Courier.  CGI program usually launched by a web server.
#        Contacts an iris RAD, gets port number of available iris REX, 
#        contacts REX, passes cgi data, waits for response, passes response
#        back to web server.
#
# Params: Requires a configuration file for all parameters.
#****************************************************************************

#binmode(STDIN);
#binmode(STDOUT);
#binmode(STDERR);

&Parse_Data;

# Read configuration data, strip out comment lines and store
# the name value pairs in a hash named params
unless(open(TMP, "< iris.cfg")) { die "Could not read configuration file."; }
@lines = <TMP>;
unless(close(TMP)) { die "Could not close configuration file."; }
@lines = grep !/^#/, @lines;
for (@lines) {
  chomp(($key,$value) = split('=',$_,2));
  $params{$key} = $value;
}

$RadHost = $params{'RadHost'};
$RadPort = $params{'RadPort'};
$RadTimeout = $params{'RadTimeout'};
$RexTimeout = $params{'RexTimeout'};

use IO::Socket;
use IO::Select;

unless($sock = IO::Socket::INET->new("$RadHost:$RadPort")) {
  $RedirPage = $params{'NoRadErr'};
  &ErrReDir;
}

$sock->autoflush(1);

$sel = IO::Select->new();
$sel->add($sock);

print $sock "Iris: Req \n";

@ready = $sel->can_read($RadTimeout);

if (@ready) {
  chomp($RexPort = <$sock>);
  $sock->close;
} else {
  $RedirPage = $params{'RadTimeoutErr'};
  &ErrReDir;
}

$sel->remove($sock);

if ($RexPort) {

  $sock = IO::Socket::INET->new("$RadHost:$RexPort");
  unless ($sock) {
    $RedirPage = $params{'NoRexErr'};
    &ErrReDir;
  }

  $sock->autoflush(1);

  $sel = IO::Select->new();
  $sel->add($sock);

  print $sock 
"AUTH_TYPE=$ENV{'AUTH_TYPE'}
CONTENT_LENGTH=$ENV{'CONTENT_LENGTH'}
CONTENT_TYPE=$ENV{'CONTENT_TYPE'}
GATEWAY_INTERFACE=$ENV{'GATEWAY_INTERFACE'}
HTTP_ACCEPT=$ENV{'HTTP_ACCEPT'}
HTTP_ACCEPT_LANGUAGE=$ENV{'HTTP_ACCEPT_LANGUAGE'}
HTTP_COOKIE=$ENV{'HTTP_COOKIE'}
HTTP_REFERER=$ENV{'HTTP_REFERER'}
HTTP_USER_AGENT=$ENV{'HTTP_USER_AGENT'}
HTTPS=$ENV{'HTTPS'}
PATH_INFO=$ENV{'PATH_INFO'}
PATH_TRANSLATED=$ENV{'PATH_TRANSLATED'}
QUERY_STRING=$ENV{'QUERY_STRING'}
REMOTE_ADDR=$ENV{'REMOTE_ADDR'}
REMOTE_HOST=$ENV{'REMOTE_HOST'}
REMOTE_IDENT=$ENV{'REMOTE_IDENT'}
REMOTE_USER=$ENV{'REMOTE_USER'}
REQUEST_METHOD=$ENV{'REQUEST_METHOD'}
SCRIPT_NAME=$ENV{'SCRIPT_NAME'}
SERVER_NAME=$ENV{'SERVER_NAME'}
SERVER_PORT=$ENV{'SERVER_PORT'}
SERVER_PROTOCOL=$ENV{'SERVER_PROTOCOL'}
SERVER_SOFTWARE=$ENV{'SERVER_SOFTWARE'}\n";

  foreach $s (keys(%CGI)) {
      print $sock "$s=$CGI{$s}\n";
  }
  print $sock "\n";

  while (1) {
    if(@ready = $sel->can_read($RexTimeout)) {
      $temp = readline($sock);
      last if ! $temp;
      print "$temp";
    } else {
      print "</br></br><h3>The host application seems to have stopped sending data. \n";
      print "Please try your request again or contact the system administrator.</h3> \n";
      print "</body></html>\n";
      exit;
    }
  }

  $sock->close; 

} else {
  $RedirPage = $params{'NoRexAvailErr'};
  &ErrReDir;
}

sub ErrReDir {
  print "Status: 301\n";
  print "Content-Type: text/html\n";
  print "Location: $RedirPage\n\n";
  exit;
}

sub Parse_Data {
    local($raw_data,@items,$key,$value);

    $raw_data = &Parse_Method;

    # Split different "NAME" and their values.
    @items = split('&', $raw_data);

    # For each list of "NAME=its_value".
    for (@items) {

        $_ =~ tr/+/ /;
        ($key,$value) = split('=',$_,2);

        # The %xx hex numbers are converted to alphanumeric.
        $key   =~ s/%(..)/pack("C", hex($1))/eg;
        $value =~ s/%(..)/pack("C", hex($1))/eg;
        if (grep(/^$key$/, keys(%CGI))) {
	  if (@{$CGI{$key}} > 0) {
            push(@{$CGI{$key}}, "$value");
          }
	  else {
	    $arrvalue = $CGI{$key};
	    undef $CGI{$key};
            $CGI{$key}[0] = $arrvalue;
	    push(@{$CGI{$key}}, "$value");
          }
        }
        else {
          $CGI{$key} = $value;
        }
     }
}

sub Parse_Multi {
    local($boundary,@pairs,$position);
    local($raw_data,$value,$name,$part);

    $raw_data = &Parse_Method;
    ($boundary = $ENV{CONTENT_TYPE}) =~ s/^.*boundary=(.*)$/\1/;
    @pairs = split(/--$boundary/, $raw_data);
    @pairs = splice(@pairs,1,$#pairs-1);

    for $part (@pairs) {
      $part =~ s/[\r]\n$//g;
      ($dump, $firstline, $datas) = split(/[\r]\n/, $part, 3);
      next if $firstline =~ /filename=\"\"/;
      $firstline =~ s/^Content-Disposition: form-data; //;
      (@columns) = split(/;\s+/, $firstline);
      ($name = $columns[0]) =~ s/^name="([^"]+)"$/\1/g;
      if ($#columns > 0) {
	if ($datas =~ /^Content-Type:/) {
	  ($CGI{"$name"}->{'Content-Type'}, $blankline, $datas) = split(/[\r]\n/, $datas, 3);
          $CGI{"$name"}->{'Content-Type'} =~ s/^Content-Type: ([^\s]+)$/\1/g;
	}
	else {
	  ($blankline, $datas) = split(/[\r]\n/, $datas, 2);
	  $CGI{"$name"}->{'Content-Type'} = "application/octet-stream";
	}
      }
      else {
	($blankline, $datas) = split(/[\r]\n/, $datas, 2);
        if (grep(/^$name$/, keys(%CGI))) {
          if (@{$CGI{$name}} > 0) {
            push(@{$CGI{$name}}, $datas);
          }
          else {
            $arrvalue = $CGI{$name};
            undef $CGI{$name};
            $CGI{$name}[0] = $arrvalue;
            push(@{$CGI{$name}}, $datas);
          }
        }
        else {
	  next if $datas =~ /^\s*$/;
          $CGI{"$name"} = $datas;
        }
        next;
      }
      for $currentColumn (@columns) {
        ($currentHeader, $currentValue) = $currentColumn =~ /^([^=]+)="([^"]+)"$/;
        $CGI{"$name"}->{"$currentHeader"} = $currentValue;
      }
      $CGI{"$name"}->{'Contents'} = $datas;
    }
}

sub Parse_Method {
    local($buffer);

    if ($ENV{'REQUEST_METHOD'} eq "POST") {
      read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
      return $buffer;
    }
    elsif ($ENV{'REQUEST_METHOD'} eq "GET") {
      return $ENV{'QUERY_STRING'};
    }
    else {
      return 0;
    }
}