/**************************************************************************
The contents of this file are subject to the POSSENET Public License 
Version 1.0 (the "License"); you may not use this file except in compliance 
with the License. You may obtain a copy of the License at 
http://www.possenet.org/license.html.

Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
for the specific language governing rights and limitations under the 
License. 

The Original Code is Progress IDE, released December 1, 2000. 

The Initial Developer of the Original Code is Progress Software 
Corporation. Portions created by are Copyright (c) 2000. All Rights 
Reserved. 
**************************************************************************/


function output-http-header returns char 
  (input vc_hdr as char, input vc_val as char).
  
  if output-content-type = ? then do:
    vc_hdr = right-trim(vc_hdr,": ").
    if vc_hdr > "" then vc_hdr = vc_hdr + ": ".
  
    {&out} vc_hdr vc_val skip.
  end.
end function.

function output-content-type return char
  (input vc_type as char).
  
  if vc_type > "" and output-content-type = ? then do:
    output-content-type = vc_type.
    {&out} "Content-Type: " output-content-type skip(1).
  end.
  
end function.

function get-value returns char (input vc_key as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "user" no-lock no-error.
  if avail cgi then return cgi.val.
  find first cgi where cgi.key = vc_key and
    cgi.type = "field" no-lock no-error.
  if avail cgi then return cgi.val.
  find first cgi where cgi.key = vc_key and
    cgi.type = "cookie" no-lock no-error.
  if avail cgi then return cgi.val.
  return "".
end function.

function get-field return char (input vc_key as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "field" no-lock no-error.
  if avail cgi then return cgi.val.
  return "".
end function.

function get-cookie return char (input vc_key as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "cookie" no-lock no-error.
  if avail cgi then return cgi.val.
  return "".
end function.

function get-user-field return char (input vc_key as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "user" no-lock no-error.
  if avail cgi then return cgi.val.
  return "".
end function.

function get-cgi return char (input vc_key as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "cgi" no-lock no-error.
  if avail cgi then return cgi.val.
  return "".
end function.

function set-user-field return char (input vc_key as char, vc_val as char).
  find first cgi where cgi.key = vc_key and
    cgi.type = "user" exclusive no-error.
  if not avail cgi then do:
    create cgi.
    assign
      cgi.key = vc_key
      cgi.type = "user".
  end.
  cgi.val = vc_val.
end function.

FUNCTION convert-datetime RETURNS CHARACTER
  (INPUT p_conversion AS CHARACTER,
   INPUT p_idate       AS DATE,
   INPUT p_itime       AS INTEGER,
   OUTPUT p_odate      AS DATE,
   OUTPUT p_otime      AS INTEGER) :

  DEFINE VARIABLE seconds-per-day AS INTEGER NO-UNDO INITIAL 86400.

  /* Default option is to normalize */
  IF p_conversion = "" OR p_conversion = ? THEN
    ASSIGN p_conversion = "NORMALIZE":U.

  /* If date is ? ... */
  IF p_idate = ? THEN DO:
    RETURN "".
  END.

  IF p_itime = ? THEN
    ASSIGN p_itime = 0.

  /* Set time adjustment depending on conversion option */
  CASE p_conversion:
    WHEN "LOCAL":U THEN
      ASSIGN p_itime = p_itime - utc-offset.
    WHEN "UTC":U THEN
      ASSIGN p_itime = p_itime + utc-offset.
  END CASE.

  /* Normalize if time is too large */
  DO WHILE p_itime >= seconds-per-day:
    ASSIGN
      p_itime = p_itime - seconds-per-day
      p_idate = p_idate + 1.  /* tomorrow */
  END.

  /* Normalize if time is too small */
  DO WHILE p_itime < 0:
    ASSIGN
      p_itime = p_itime + seconds-per-day
      p_idate = p_idate - 1.  /* yesterday */
  END.

  ASSIGN
    p_odate = p_idate
    p_otime = p_itime.
  RETURN "".

END FUNCTION. /* convert-datetime */

FUNCTION format-datetime RETURNS CHARACTER
  (INPUT p_format AS CHARACTER,
   INPUT p_date   AS DATE,
   INPUT p_time   AS INTEGER,
   INPUT p_options AS CHARACTER) :

  DEFINE VARIABLE p_rfcdate AS CHARACTER NO-UNDO.

  /* Does RFC 850/822 allow translated days and months?  Think not. */
  DEFINE VARIABLE weekday-list AS CHARACTER NO-UNDO INITIAL
    "Sun,Mon,Tue,Wed,Thu,Fri,Sat":U.
  DEFINE VARIABLE month-list AS CHARACTER NO-UNDO INITIAL
    "Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec":U.

  /* If date is ?, return a blank date */
  IF p_date = ? THEN
    RETURN "".

  IF p_time = ? THEN
    ASSIGN p_time = 0.

  /* If no options are specified, LOCAL is the default */
  IF p_options = "" OR p_options = ? THEN
    ASSIGN p_options = "LOCAL":U.

  /* If Local was specified and the format is Cookie or HTTP, convert date and 
     time to UTC. */
  IF CAN-DO(p_options, "LOCAL":U) AND
    (p_format = "COOKIE":U OR p_format = "HTTP":U) THEN
    /* Convert date and time from Local to UTC */
    convert-datetime(INPUT "UTC":U,
                     INPUT p_date, INPUT p_time,
                     OUTPUT p_date, OUTPUT p_time).
  /* Otherwise, just normalize */
  ELSE
    /* Normalize date and time */
    convert-datetime(INPUT "NORMALIZE":U,
                     INPUT p_date, INPUT p_time,
                     OUTPUT p_date, OUTPUT p_time).

  /* Output the formatted date */
  CASE p_format:
    WHEN "COOKIE":U THEN DO:
      /* Cookie format based on RFC-1123: Wdy, DD-Mon-YYYY HH:MM:SS GMT */
      ASSIGN 
        p_rfcdate = ENTRY(WEEKDAY(p_date), weekday-list) + ", ":U +
                    STRING(DAY(p_date),"99":U) + "-":U +
                    ENTRY(MONTH(p_date), month-list) + "-":U +
                    STRING(YEAR(p_date), "9999":U) + " ":U +
                    STRING(p_time,"HH:MM:SS":U) + " GMT":U.
    END.
    WHEN "HTTP":U THEN DO:
      /* HTTP format based on RFC-1123: Wdy, DD Mon YYYY HH:MM:SS GMT */
      ASSIGN 
        p_rfcdate = ENTRY(WEEKDAY(p_date), weekday-list) + ", ":U +
                    STRING(DAY(p_date),"99":U) + " ":U +
                    ENTRY(MONTH(p_date), month-list) + " ":U +
                    STRING(YEAR(p_date), "9999":U) + " ":U +
                    STRING(p_time,"HH:MM:SS":U) + " GMT":U.
    END.
/*
    OTHERWISE
      queue-message("WebSpeed":U, "format-datetime: format '" + p_format +
                    "' is not supported").
*/

  END CASE.
  RETURN p_rfcdate.
END FUNCTION. /* format-datetime */

FUNCTION url-encode RETURNS CHARACTER
  (INPUT p_value AS CHARACTER,
   INPUT p_enctype AS CHARACTER) :

  DEFINE VARIABLE hx          AS CHARACTER NO-UNDO INITIAL "0123456789ABCDEF":U.
  DEFINE VARIABLE encode-list AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i           AS INTEGER   NO-UNDO.
  DEFINE VARIABLE c           AS INTEGER   NO-UNDO.
 
  /* Don't bother with blank input */
  IF LENGTH(p_value) = 0 OR p_value = ? THEN
    RETURN "".

  IF p_enctype = ? THEN
    ASSIGN p_enctype = "":U.

  /* What kind of encoding should be used? */
  CASE p_enctype:
    WHEN "query":U THEN              /* QUERY_STRING name=value parts */
      encode-list = url_unsafe + url_reserved + "+":U.
    WHEN "cookie":U THEN             /* Persistent Cookies */
      encode-list = url_unsafe + " ,~;":U.
    WHEN "default":U OR WHEN "" THEN /* Standard URL encoding */
      encode-list = url_unsafe.
    OTHERWISE
      encode-list = url_unsafe + p_enctype.   /* user specified ... */
  END CASE.

  /* Loop through entire input string */
  ASSIGN i = 0.
  DO WHILE TRUE:
    ASSIGN
      i = i + 1  /* Next character */
      /* ASCII value of character */
      c = ASC(SUBSTRING(p_value, i, 1, "RAW":U)).
    IF c <= 31 OR c >= 127 OR INDEX(encode-list, CHR(c)) > 0 THEN DO:
      /* Replace character with %hh hexidecimal triplet */
      SUBSTRING(p_value, i, 1, "RAW":U) =
        "%":U +
        SUBSTRING(hx, INTEGER(TRUNCATE(c / 16, 0)) + 1, 1, "RAW":U) + /* high */
        SUBSTRING(hx, c MODULO 16 + 1, 1, "RAW":U).             /* low digit */
      ASSIGN i = i + 2.   /* skip over hex triplet just inserted */
    END.
    IF i = LENGTH(p_value,"RAW") THEN LEAVE.
  END.

  RETURN p_value.
END FUNCTION.  /* url-encode */

FUNCTION set-cookie RETURNS CHARACTER
 (INPUT p_name   AS CHARACTER,
  INPUT p_value  AS CHARACTER,
  INPUT p_date   AS DATE,
  INPUT p_time   AS INTEGER,
  INPUT p_path   AS CHARACTER,
  INPUT p_domain AS CHARACTER,
  INPUT p_options AS CHARACTER):

  DEFINE VARIABLE exp-date      AS CHARACTER NO-UNDO.
  DEFINE VARIABLE v-cookie      AS CHARACTER NO-UNDO.
  DEFINE VARIABLE v-secure      AS CHARACTER NO-UNDO.
  DEFINE VARIABLE v-timeconv    AS CHARACTER NO-UNDO.

  /* Nothing to do if no Cookie name */
  IF p_name = ? OR p_name = "" THEN RETURN "".

  IF p_value = ? THEN
    ASSIGN p_value = "".     /* ? and "" for the value are the same thing */
  IF p_path = ? THEN
    ASSIGN p_path = SCRIPT_NAME.  /* default to URL of application */
  IF p_domain = ? THEN
    ASSIGN p_domain = "".  /* From config information */
  IF p_options = ? THEN
    ASSIGN p_options = "".

  /* If any options were specified, check for "secure" or "UTC" */
  IF p_options <> "" THEN
    ASSIGN
      /* Look for "SECURE" in options */
      v-secure = (IF CAN-DO(p_options, "SECURE":U) THEN "SECURE":U ELSE "")
      /* Look for "UTC" in options. If not found, assume local time */
      v-timeconv = (IF CAN-DO(p_options, "UTC":U) THEN "UTC":U ELSE "LOCAL":U).

  /* IE and Netscape don't report cookies with null values the same way
     so it is better to expire a cookie with a null value */
  IF p_value = "" THEN
    ASSIGN p_date = DATE(1,2,1991).

  /* If an expiration date/time was specified, convert date and time to
     a modified RFC-1123 format for output. */
  IF p_date <> ? THEN DO:
    ASSIGN exp-date = format-datetime("cookie":U, p_date, p_time, v-timeconv).
    /* Set path to "/" to work around a Netscape 1.1 and earlier bug
       with expires.  This test might not be adequate. */
    IF HTTP_USER_AGENT BEGINS "Mozilla/1.1":U THEN
      p_path = "/":U.
  END.
  /* No date specified */
  ELSE
    ASSIGN exp-date = "".

  /* Format the cookie */
  ASSIGN v-cookie =
    url-encode(p_name,"cookie":U) + "=":U +
    url-encode(p_value,"cookie":U) +
    (IF exp-date = "" THEN "" ELSE "~; expires=":U + exp-date) +
    (IF p_path = "" THEN "" ELSE "~; path=":U + p_path) +
    (IF p_domain = "" THEN "" ELSE "~; domain=":U + p_domain) +
    (IF v-secure = "" THEN "" ELSE "~; secure":U).

  /* Send the cookie to the web browser */
  output-http-header("Set-Cookie":U, v-cookie).
  
  RETURN v-cookie.
END FUNCTION.  /* set-cookie */

FUNCTION delete-cookie RETURNS CHARACTER
  (INPUT p_name AS CHARACTER,
   INPUT p_path AS CHARACTER,
   INPUT p_domain AS CHARACTER) :
  /* Output a Set-Cookie header with a very early expiration date.
     Specify date is UTC time since there's no point in converting it. */
  set-cookie (p_name, "", DATE(1,2,1991), 0, p_path, p_domain, "UTC":U).
END FUNCTION.  /* delete-cookie */
