#! /usr/bin/perl

use IO::Socket;
use IO::Select;

# Read configuration data, strip out comment lines and store
# the name value pairs in a hash named params
unless(open(TMP, "< iris.cfg")) { die "Could not read configuration file."; }
@lines = <TMP>;
unless(close(TMP)) { die "Could not close configuration file."; }
@lines = grep !/^#/, @lines;
for (@lines) {
  chomp(($key,$value) = split('=',$_,2));
  $params{$key} = $value;
}

# If max and min got reversed fix it
if ($params{'RexMax'} < $params{'RexMin'}) {
  $temp = $params{'RexMax'};
  $params{'RexMax'} = $params{'RexMin'};
  $params{'RexMin'} = $temp;
}

# If the init value is greater than the ports available, fix it
if ((($params{'RexMax'} - $params{'RexMin'}) + 1) < $params{'RexInit'}) {
  $params{'RexInit'} = (($params{'RexMax'} - $params{'RexMin'}) + 1);
}

# Start the RAD server
$server = new IO::Socket::INET(Listen => 10, 
                               LocalPort => $params{'RadPort'},
                               Reuse => 1)
  or die "Could not establish a server on port $params{'RadPort'} : $@\n";

print "Server stated on $params{'RadPort'}\n";

# Just a little info to display
print "RexMax is $params{'RexMax'}\n";
print "RexLwm is $params{'RexLwm'}\n";

# Start the initial REX sessions
for ($temp = 0; $temp < $params{'RexInit'}; $temp++) {
  $RexPort = ($params{'RexMin'} + $temp);
  &StartRex;
}

# Wait for connection requests, read the message and see if it begins
# with a recognized string
while ($client = $server->accept()) {
  $message = <$client>;
  if (substr($message,0,10) eq "Iris: Req ") { &ReqAdd; }
  elsif (substr($message,0,10) eq "Rex: Avail") { &RexAvail; }
  elsif (substr($message,0,10) eq "Cmd: Stat ") { &RexStat; }
  elsif (substr($message,0,10) eq "Cmd: Stop ") { &StopAll; }
  else { $client->close; }
}

# Sub to start REX sessions
sub StartRex {
  if ($RexPort <= $params{'RexMax'}) {   
    print "Starting Rex on $RexPort\n";

    if ($^O == "MSWin32") {
      use Win32::Process; 
      use Win32;

      Win32::Process::Create($ProcessObj,
                              "$params{'RexExe'}",
                              " $params{'RexArg'} $params{'RadPort'}:$RexPort",
                              0,
                              NORMAL_PRIORITY_CLASS,
                              ".");
    } else {
      system ("nohup $params{'RexExe'} $params{'RexArg'} $params{'RadPort'}:$RexPort > /dev/null &");
    }
  }
}
  
# Sub to update a REX's status to "Available"
sub RexAvail {
  chomp($RexAddr = substr($message,10));
  push (@RexAvail, $RexAddr);
  $RexArray{$RexAddr} = "Available";
  &RexAlloc;
}

# Sub to compare the request array to the available REX array
# and hand out REX ports to the waiting Iris instances
sub RexAlloc {
  $temp1 = @RexAvail;
  $temp2 = @IrisReq;
  while (@RexAvail > 0 && @IrisReq > 0) {
    $IrisPort = shift @IrisReq;
    next if ! (@ready = IO::Select->new($IrisPort)->can_write(0));
    $RexAddr = shift @RexAvail;    
    print $IrisPort "$RexAddr\n";
    $IrisPort->close;
    $RexArray{$RexAddr} = "Busy";
  }
}  

sub RexRem {

}

# Sub to add requests to the request array
sub ReqAdd {
  push (@IrisReq, $client);
  &RexAlloc;
  if (@RexAvail <= $params{'RexLwm'} && $RexPort < $params{'RexMax'}) {
    $RexPort++;
    &StartRex;
  }
}

# Sub to stop all REX instances
sub StopAll {
  $server->close;
  while (@IrisReq > 0) {
    $IrisPort = shift @IrisReq;
    print $IrisPort "\n";
    $IrisPort->close;
  }
  while (@RexAvail > 0) {
    $RexAddr = shift @RexAvail;
    $sock = IO::Socket::INET->new("localhost:$RexAddr");
    select($sock);
    $| = 1;
    select (STDOUT);
    print $sock "*quit*quit*quit*\n\n";
    $sock->close;
  }
  exit;
}

# Sub to report the status of all known REX instances
sub RexStat {
  foreach $s (keys(%RexArray)) {
      last if ! (@ready = IO::Select->new($client)->can_write(0));
      print $client "$s $RexArray{$s}\n";
  }
  $client->close;
}